/* Copyright Nicolas GUEDON - November 2015 - FabLab Lannion Ker Nel
 * Description: Switch On and Shutdown a target computer with a remote control
 * Spec: 
 * - Arduino with infrared receiver (to read the key of the remote control
 * - Ethernet shield to send packet over local lan
 * - Send magic packet over local lan to a target computer to switch on it 
 * - Check that computer can receive and wake on lan
 * - If arduino is switch off (electric unplug), user don't need to reconfigure it (only need to reconfigure if the target computer change or remote control key change)
 * 
 * Install:
 *  - Plug arduino to ethernet (local) then plug electric
 *  - Copy exec file WolAndShut.exe on your computer and install it on boot folder (to boot at startup)
 *  - Execute the WolAndShut.exe
 *  - Launch installation procedure on arduino side:
 *        . Pull down the switch to learn mac adress of distant computer (switch on IR side)
 *        . Wait the green led finish to blink (goes off)
 *        . Pull down the switch to learn remote code
 *        . When the green led is on (no blinking) To the arduino direction with your remote control put the key for wake on lan
 *        . Wait one seconds (green led is blinking)
 *        . Wait for the green led stay on (not blinking) and press the key for shutdown
 *        . Wait few seconds, the led is blinking and then goes off.
 *        . The configuration is finished.
 *        
 *  Todo:
 *  - Code optimisation
 *  - Create a linux release of python code
 */

#include <IRremote.h> // Specific library to receive infrared control code
#include <EtherCard.h>
#include <IPAddress.h>
#include <EEPROM.h>

// Declaration pour la partie Ethernet
// Activation DHCP
#define STATIC 0
// L'adresse mac de l'arduino
static byte mymac[] = { 0x70,0x69,0x69,0x2D,0x30,0x31 };
// L'adresse mac du pc cible (automatiser, demarrer arduino avant le pc, avant le script python)
static byte destmac[6];
byte Ethernet::buffer[500];
uint16_t ethPort = 5001; // Le port d'ecoute et d'envoi
// Pour le shutdown
char keyword[] = {"shutdown"};
byte ipDestination[] = {255, 255, 255, 255};
unsigned int srcPort = 5002;
unsigned int portDestination = 5003; 
// Fin declaration partie ethernet

 // Declaration pour la partie infrarouge
 int IR_PIN = 3; //Pin de la reception infrarouge
 IRrecv irrecv(IR_PIN);
 decode_results IRCODE; //C'est le code reçu venant de la telecommande
 unsigned long codeRecu = 0;
 // Les codes et fonctions associes (code par défaut mais sont configurable sans ecriture soft)
 unsigned long codeWOL = 11111111;
 unsigned long codeShut = 22222222;
// Fin declaration partie infrarouge

// Set led on pin A5 (pin 19 on digital mode) and variable to blink
int ledpin = 7;
int macRStop = 1;
int codeWolStop = 1;
int codeShutStop = 1;
// Set switches pins
int codeSwitchMaster = 8;
int codeSwitchSlave = 9;
int macSwitchMaster = 4;
int macSwitchSlave = 5;


// Callback function to take mac address for the wake on lan
void udpSerialPrint(uint16_t port, uint8_t ip[4], uint16_t src_port, const char *data, uint16_t len) {
  Serial.println(data);
  //Convert mac adress to byte array to sendWol
  int values[6];
  int i;
  // Check that we received a mac address
  if( 6 == sscanf( data, "%x:%x:%x:%x:%x:%x%c", &values[0], &values[1], &values[2], &values[3], &values[4], &values[5] ) ) {
      /* convert to uint8_t */
      for( i = 0; i < 6; ++i ) {
         destmac[i] = (byte) values[i]; // destmac is now updated
      }
      // Enregistrement de la mac cible en EEPROM
      EEPROM.put(0,destmac);
      macRStop = 0;
  }
  
}

void setup() {

  EEPROM.get(0, destmac); //On recupere l'adresse mac qui est enregistre EEPROM
  EEPROM.get(100, codeWOL); // On recupere le code telecommande de WOl situee en EEPROM
  EEPROM.get(500, codeShut); // On recupere le code telecommande de Shutdown situee en EEPROM
  Serial.begin(9600); // Port serie pour debug
  pinMode(ledpin, OUTPUT); //Pin led as output
  digitalWrite(ledpin, LOW);
  pinMode(codeSwitchMaster, OUTPUT);
  digitalWrite(codeSwitchMaster, LOW);
  pinMode(codeSwitchSlave, INPUT_PULLUP);
  pinMode(macSwitchMaster, OUTPUT);
  digitalWrite(macSwitchMaster, LOW);
  pinMode(macSwitchSlave, INPUT_PULLUP);
  Serial.println("Start!");
  digitalWrite(ledpin, HIGH);
  irrecv.enableIRIn(); // Start the infrared receiver
  // Declare ethernet card
  if (ether.begin(sizeof Ethernet::buffer, mymac, 10) == 0) // Port 10
  Serial.println( "Failed to access Ethernet controller");
  if (!ether.dhcpSetup()) // Si problème de DHCP ca ne peut pas fonctionner
  Serial.println("DHCP failed");
  // Display ethernet ip details
  ether.printIp("IP: ", ether.myip);
  ether.printIp("GW: ", ether.gwip);
  ether.printIp("DNS: ", ether.dnsip);
  // Open port to listen on
  ether.udpServerListenOnPort(&udpSerialPrint, ethPort);
  digitalWrite(ledpin, LOW);
}

void loop() {
  // Launch ethernet card to listen (udpServer on setup)
  ether.packetLoop(ether.packetReceive());
  
  //IR receiving part
  if (irrecv.decode(&IRCODE)) {
    Serial.println(IRCODE.value);
    codeRecu = IRCODE.value;
    digitalWrite(ledpin, HIGH);
    delay(10);
    digitalWrite(ledpin, LOW);
    irrecv.resume(); // Receive the next value
  }
  delay(10);
  // End of IR receiving part
  
  // Switch code remote control part
  if(digitalRead(codeSwitchSlave) == LOW) {
    Serial.println("RequestTheWol IR remote code");
    for(int j=0; j<5; j++) {
      digitalWrite(ledpin, HIGH);
      delay(100);
      digitalWrite(ledpin,LOW);
      delay(100);
    }
    int i = 0;
    digitalWrite(ledpin, HIGH);
    while(codeWolStop) {
      if (irrecv.decode(&IRCODE)) {
        Serial.print("Ancien code: ");
        Serial.println(codeWOL);
        codeWOL = IRCODE.value;
        EEPROM.put(100,codeWOL);
        codeWolStop = 0;
        delay(10);
        Serial.print("Nouveau code: ");
        Serial.println(codeWOL);
        delay(300);
        irrecv.resume();
      }
      if(i == 2000) { //Timeout if no code is received
       codeWolStop = 0;
      }
      i++;
      delay(10);
    }
    codeWolStop = 1;
    digitalWrite(ledpin, LOW);
    delay(1000);
    Serial.println("RequestTheShut IR remote code");
    for(int j=0; j<5; j++) {
      digitalWrite(ledpin, HIGH);
      delay(100);
      digitalWrite(ledpin,LOW);
      delay(100);
    }
    i = 0;
    digitalWrite(ledpin, HIGH);
    while(codeShutStop) {
      if (irrecv.decode(&IRCODE)) {
        Serial.print("Ancien code: ");
        Serial.println(codeShut);
        codeShut = IRCODE.value;
        EEPROM.put(500,codeShut);
        codeShutStop = 0;
        delay(10);
        Serial.print("Nouveau code: ");
        Serial.println(codeShut);
        delay(300);
        irrecv.resume();
      }
      if(i == 2000) { //Timeout if no response is received
       codeShutStop = 0;
      }
      i++;
      delay(10);
    }
    codeShutStop = 1;
    digitalWrite(ledpin, LOW);
    delay(500);
    for(int j=0; j<5; j++) {
      digitalWrite(ledpin, HIGH);
      delay(500);
      digitalWrite(ledpin,LOW);
      delay(500);
    }
  }
  //End of switch remote control code

  // Switch mac setting, send an udp with keyword to request the mac address of computer that is listening on specific port
  if(digitalRead(macSwitchSlave) == LOW) {
    Serial.println("RequestTheMac");
    char yourmac[] = {"yourmac?"};
    ether.sendUdp(yourmac, sizeof yourmac, srcPort, ipDestination, portDestination);
    int i = 0;
    while(macRStop) {
      digitalWrite(ledpin, HIGH);
      delay(500);
      digitalWrite(ledpin, LOW);
      delay(500);
      ether.packetLoop(ether.packetReceive());
      if(i == 200) { //Timeout if no response is received
       macRStop = 0;
      }
      i++;
    }
    macRStop = 1;
  }
  //End of switch mac setting
  
  
  

  // Action to do link to IRcode
  if(codeRecu == codeWOL) { //Action Wake On Lan
    codeRecu = 0; // On reinitialise le code
    Serial.println("Wake up!!!");
    ether.sendWol(destmac); //Envoi d'un magic packet a l'adresse mac cible
  } else if (codeRecu == codeShut) { // Action shutdown
    codeRecu = 0; // On reinitialise le code
    //Shutdown ici
    Serial.println("Time to shutdown!!!");
    ether.sendUdp(keyword, sizeof keyword, srcPort, ipDestination, portDestination);
  } else { 
    codeRecu = 0; // On reinitialise le code
    //Autre cas, on ne fait rien
  }
  // End of action to do link to IRcode

}

